#define  F_CPU 4000000UL
//#define  F_CPU 8000000UL
//#define  F_CPU 16000000UL
//#define  F_CPU 14745600UL

#include <avr/io.h>
#include <avr/pgmspace.h>

void UART_Init(unsigned long baud)
{
	/*1 - Set the agreed on Baud Rate between Sender & Receiver by writing to UBRRL/UBRRH*/
	unsigned short UBRR;  //You put the suitable value to achieve required Baud
	UBRR=(F_CPU/(16*baud))-1;  //Equation explained in data sheet
	UBRRH=(unsigned char)(UBRR>>8);  //Put the MSB of UBRR into UBRRH.  Used Casting because taking from Short to Char (8 bits)
	UBRRL=(unsigned char)UBRR;  //Put LSB of UBRR into UBRRL.  Used Casting because taking from Short to Char (8 bits)
	/*2 - Enable USART Sender & Receiver*/
	UCSRB = (1<<RXEN)|(1<<TXEN);
	/*3 - Configure Character size, parity, stop bits
	, We will work with 8 bits data,1 stop bit and no parity bits*/
	UCSRB&=(~(1<<UCSZ2)); //For 8-bits  -  default
	UCSRC=0b10000110;  //Asynchronous mode, No Parity, 1 stop bit, 8bits
}

void UART_SendChar(char data)
{
	/*Wait until UDR transmit buffer is empty*/
	//while ( ((UCSRA&(1<<UDRE))>>UDRE )==0);
	while(!(UCSRA & (1<<UDRE)));
	//while(READ_BIT(UCSRA,UDRE)==0);
	/*Put data to UDR transmit buffer & transmit*/
	UDR=data ;
}

char UART_RecvChar(void)
{
	/*Wait for UDR receive buffer is filled with data*/
	//while ( ((UCSRA&(1<<RXC))>>RXC )==0);
	while ((UCSRA & (1 << RXC)) == 0);
	//while(READ_BIT(UCSRA,RXC)==0);
	/*Receive data from UDR receive buffer*/
	return UDR ;
}

void UART_SendString( char *ptr)
{
	//Put the string into a pinter to character then move the pointer to the next character and send it
	while(*ptr!=0)
	{
		UART_SendChar(*ptr);
		ptr++;
	}
}

void UART_SendString_p( char *ptr)
{
	//Put the string into a pinter to character then move the pointer to the next character and send it
	while(pgm_read_byte (ptr)!=0x00)
	{
		UART_SendChar(pgm_read_byte (ptr++));
	}
}